using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;

namespace PrivateInvestigationNetworkTool
{
    /// <summary>
    /// implements the packet test tool - NO error checkings yet
    /// </summary>
    public partial class PacketTool : Form
    {
        System.Net.Sockets.TcpClient client;
        NetworkStream stream;
        string message;
        Byte[] data;
        String responseData;

        public PacketTool()
        {
            InitializeComponent();
            protocoltype.SelectedIndex = 1;
        }

        // taken from http://authors.aspalliance.com/olson/methods/GetStandardDeviation.aspx
        public static double GetStandardDeviation(double[] num)
        {
            double Sum = 0.0, SumOfSqrs = 0.0;
            for (int i = 0; i < num.Length; i++)
            {
                Sum += num[i];
                SumOfSqrs += Math.Pow(num[i], 2);
            }
            double topSum = (num.Length * SumOfSqrs) - (Math.Pow(Sum, 2));
            double n = (double)num.Length;
            return Math.Sqrt(topSum / (n * (n - 1)));
        }

        private void EchoTest(int port,int iterations,System.Net.Sockets.ProtocolType useThisProtocol)
        {
            #region Initializations
            long test_Start_Time = System.Environment.TickCount;
            double[] TimeTable = new double[iterations+1];
            #endregion

            if (useThisProtocol == ProtocolType.Udp)
            {
                #region the UDP echo test
                // TODO: implement it
                UdpClient testclient = new UdpClient(ip_adress.Text, port);

                #endregion
            }
            else
            {
                #region the TCP echo test
                TcpClient testclient = new TcpClient(ip_adress.Text, port);
                NetworkStream teststream = testclient.GetStream();

                #region create EchoServer
                // start an echoserver
                message = "start echoserver " + port + "/tcp\n";

                // Translate the passed message into ASCII and store it as a Byte array.
                data = System.Text.Encoding.ASCII.GetBytes(message);

                stream.Write(data, 0, data.Length);

                waitfordataanddisplay(new TimeSpan(0, 0, 10));
                #endregion

                #region iterations
                message = "Nothing here but zeroes.                                                                  \n";
                data = System.Text.Encoding.ASCII.GetBytes(message);
                for (int iterationsdone = 0; iterationsdone <= iterations; iterationsdone++)
                {
                    #region send a packet and wait for it's return
                    // set reference time
                    long tcp_packet_starttime = System.Environment.TickCount;
                    // send data
                    teststream.Write(data, 0, data.Length);
                    int waited=0;
                    // wait (max 5 seconds) for the data to arrive back
                    // we can wait 1 msec because the packets will unlikely arrive in
                    // shorter timeframes via GPRS/UMTS/WLAN
                    while ((!teststream.DataAvailable) & (waited <= 5000))
                    {
                        // wait...wait...wait...
                        System.Threading.Thread.Sleep(1); // 1 msec
                        waited++;
                    }
                    Int32 bytes = teststream.Read(data, 0, data.Length);

                    long tcp_packet_endtime = System.Environment.TickCount;
                    
                    TimeTable[iterationsdone] = tcp_packet_endtime - tcp_packet_starttime;
                    output_box.Text = output_box.Text + TimeTable[iterationsdone]+" ";
                    #endregion
                }
                output_box.Text = output_box.Text + "\r\n";
                teststream.Close();
                #region calculate some stuff
                double Deviation = GetStandardDeviation(TimeTable);

                output_box.Text = output_box.Text + "Standard deviation: " + Deviation + "\r\n";
                #endregion
                #endregion

                #endregion
            }
            DateTime test_End_Time = DateTime.Now;

        }

        #region Wait for Data and Display it in the Textbox
        private void waitfordataanddisplay(TimeSpan timeout)
        {
            DateTime startingtime = DateTime.Now;
            DateTime testtime = DateTime.Now;
            while ( (!stream.DataAvailable) & (startingtime-testtime <= timeout) )
            {
                testtime = DateTime.Now;
                System.Threading.Thread.Sleep(1);
            }
            if (!(startingtime - testtime <= timeout))
            {
                output_box.Text = output_box.Text + "\r\n"+ "Arr, a timeout occured." + "\r\n";
            }

            while (stream.DataAvailable)
            {
                Int32 bytes = stream.Read(data, 0, data.Length);
                responseData = System.Text.Encoding.ASCII.GetString(data, 0, bytes);

                output_box.Text = output_box.Text + responseData;

            }
        }
        #endregion

        private void mI_PingButton_Click(object sender, EventArgs e)
        {
            try
            {
            client = new TcpClient(ip_adress.Text, Convert.ToInt32(port_number.Text,10));

            // the current channel nickname...forget about it now - it has to be sent in the first place
            // before we can do anything with the echo server
            message = "set nickname 22c3\n";

            // Translate the passed message into ASCII and store it as a Byte array.
            data = System.Text.Encoding.ASCII.GetBytes(message);
                // Get a client stream for reading and writing.
                //  Stream stream = client.GetStream();

                 stream = client.GetStream();

                // Send the message to the connected TcpServer. 
                //stream.Write(data, 0, data.Length);

                // Receive the TcpServer.response.

                // Buffer to store the response bytes.
                data = new Byte[4096];

                // String to store the response ASCII representation.
                responseData = String.Empty;

                waitfordataanddisplay(new TimeSpan(0,0,10));

                data = System.Text.Encoding.ASCII.GetBytes(message);
                
                stream.Write(data, 0, data.Length);

                waitfordataanddisplay(new TimeSpan(0, 0, 10));

                // clear display
                output_box.Text = "";

                // start echoserver port/protocol
                if (protocoltype.SelectedIndex == 1)
                {
                    for (int counter = Convert.ToInt32(port_range_begin.Text, 10); counter <= Convert.ToInt32(port_range_ends.Text, 10); counter++)
                    {
                       EchoTest(counter, Convert.ToInt32(iterations.Value), ProtocolType.Tcp);
                    }
                }
                else
                    EchoTest(Convert.ToInt32(port_range_begin.Text, 10), Convert.ToInt32(iterations.Value), ProtocolType.Udp);
                // now we should have a echoserver on the port we submitted

            }
            catch (SocketException)
            {
                output_box.Text = output_box.Text + "\r\n" + "Arr, a socketexception occured." + "\r\n";
                output_box.ScrollToCaret();
            }
            
            message = "quit\n";
            data = System.Text.Encoding.ASCII.GetBytes(message);
            try
            {
                stream.Write(data, 0, data.Length);
                waitfordataanddisplay(new TimeSpan(0, 0, 10));
            }
            catch { }
        }

        private void mI_backButton_Click(object sender, EventArgs e)
        {
            try
            {
                stream.Close();
            }
            catch { }
            this.Close();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                Byte[] data = System.Text.Encoding.ASCII.GetBytes(command.Text + "\n");
                // String to store the response ASCII representation.
                String responseData = String.Empty;
                stream.Write(data, 0, data.Length);

                waitfordataanddisplay(new TimeSpan(0, 1, 0));
            }
            catch
            {
            }
        }
    }
}