package VoE;

/* (k) 2005 by ahzf voe@ahzf.de */

import java.io.*;
import java.net.*;
import java.util.*;
import java.util.StringTokenizer;
import java.util.List;
import java.util.Iterator;
import java.util.Timer;
import java.util.HashMap;
import java.io.File;
import VoE.*;



class VoE_CommServerThread extends VoE_CommServer implements Runnable {

    final  static int    BUF_SIZE = 65000;
    static final  byte[] EOL      = {(byte)'\r', (byte)'\n' };

    /* buffer to use for requests */
    byte[]	clBuf;
    byte[]	srBuf;

    /* Socket to client we're handling */
    private	Socket			clSocket;
    public	int   			threadCounter;
    private	String 			sessionCookie	= "42";
    private	String			request		= "";
    private	String			nickname	= "unknown";
    private	String			peerName	= "";
    private	VoE_CommServerThread	peer		= null;
    
    private	DataOutputStream	clOneOut;
    private	BufferedReader		clOneIn;
    private	DataOutputStream	clTwoOut;
    private	BufferedReader		clTwoIn;

    private	VoE_CommServerThread	threadTwo;
    private	int			sessionEntered	= 0;

    private	Vector			echoServers	= new Vector();




    VoE_CommServerThread(int counter) {
      clBuf		= new byte[BUF_SIZE];
      srBuf		= new byte[BUF_SIZE];
      clSocket		= null;
      threadCounter	= counter;
    }



    synchronized void setSocket(Socket clSocket) {

      this.clSocket = clSocket;

      try {
        clSocket.setSoTimeout(VoE_CommServer.timeout);
        clSocket.setTcpNoDelay(true);
        clSocket.setKeepAlive(true);
      } catch (SocketException e) {
        System.out.println("VoE Communicator --> SetSocketError at WorkerThread... "+e);
      }

      notify();

    }



    public String getNickname() {
      return(nickname);
    }



    synchronized private void setNickname(String name, VoE_CommServerThread thread) {

      boolean returnValue = VoE_SessionComm.registerNickname(name, thread);

      if (returnValue) {
        this.output("you're now known as "+name+"\r\n\r\n");
        this.nickname = name;
      } else {
        this.output("this nickname is invalid\r\n\r\n");
      }

    }



    private void setPeerName(String name) {

      if ( VoE_SessionComm.isValidPeer(name) && !nickname.equals(name) ) {
        this.output("sending peering request to "+name+"\r\n\r\n");
        VoE_SessionComm.requestPeering(nickname, name);
      } else {
        this.output("this peer is invalid\r\n\r\n");
      }

    }



    public void setPeer(VoE_CommServerThread peerThread) {

      try {

        if (peerThread != null) {
          peer = peerThread;
          this.output("peering activated\r\n\r\n");
        } else {
          System.out.println("VoE_CommServerThread.setPeer() error! Can't set peer = null.");
        }

      } catch (Exception e) {
        System.out.println("VoE_CommServerThread.setPeer() error!");
        System.out.println(e);
      }

    }



    public void disablePeering() {
      peer = null;
    }



    public void output(String Text) {
      try {
        clOneOut.writeBytes(Text);
      } catch (IOException e) {
        System.out.println("VoE Thread("+threadCounter+")        \t--> closing connection to "+clSocket.getInetAddress().getHostAddress()+", port "+clSocket.getPort());
      }
    }



    public synchronized void run() {

      while(true) {

        if (clSocket == null) {
          /* nothing to do */
          try {
            wait();
          } catch (InterruptedException e) {
            continue; /* should not happen */
          }
        }

        try {
          handleClient();
        } catch (Exception e) {
          e.printStackTrace();
        }

        /* go back in wait queue if there's fewer
         * than numHandler connections */
        clSocket = null;
        Vector pool = VoE_CommServer.threadPool;
        synchronized (pool) {
          if (pool.size() >= VoE_CommServer.maxInstances) {
            /* too many threads, exit this one */
            return;
          } else {
            pool.addElement(this);
          }
        }

      }
    }




    private void cleanup() {

      //- stop all echoservers ----------------------------------
      Enumeration enumi = echoServers.elements();

      while (enumi.hasMoreElements()) {
        String   es = (String) enumi.nextElement();
        String[] pa = es.split("/");
        VoE_SessionComm.stopEchoServer(pa[0], pa[1], nickname);
      }

      VoE_SessionComm.removeNickname(nickname);

    }



    void handleClient() throws IOException {

      String[]	parameters = {""};
      boolean	error;

      clOneOut		= new DataOutputStream(clSocket.getOutputStream()); 
      clOneIn		= new BufferedReader(new InputStreamReader(clSocket.getInputStream()));
      sessionCookie	= "sessionCookie"+clSocket.getInetAddress().getHostAddress()+clSocket.getPort();
   
      try {

        System.out.println("VoE Thread("+threadCounter+")        \t--> opening connection to "+clSocket.getInetAddress().getHostAddress()+", port "+clSocket.getPort());

        clOneOut.writeBytes("\r\n");
        clOneOut.writeBytes("\\\\     /   \r\n");
        clOneOut.writeBytes(" \\\\   /     __ \r\n");
        clOneOut.writeBytes("  \\\\ / /-/ /_  Voice-over-Everything - Communication Server v0.1 \r\n");
        clOneOut.writeBytes("   \\/ /_/ /__  ____________________________________________________ \r\n\r\n");
        clOneOut.writeBytes(" Your ip address/port: "+clSocket.getInetAddress().getHostAddress()+"/"+clSocket.getPort()+"\r\n");
        clOneOut.writeBytes(" First of all please set a nickname with \"set nickname foo\"\r\n\r\n");


        do {

          try {
            this.output(nickname+"> ");
            request	= clOneIn.readLine();
            parameters	= request.toLowerCase().split(" |/");
            error	= false;
            System.out.println(request);
          } catch (Exception e) {
            System.out.println("fucking java bug... "+e);
          }


          if (peer != null && request.startsWith("talk ")) peer.output("<"+nickname+">"+request+"</"+nickname+">\r\n");


          //- help ----------------------------------------------------------------------------
          if ( parameters.length == 1 && parameters[0].equals("help") ) {
//            VoE_SessionComm.output(sessionCookie, nickname, "the answer ist 42...\r\n");
          }


          //- set nickname $nickname ---------------------------------------------------------
          if ( parameters.length == 3 && parameters[0].equals("set") && parameters[1].equals("nickname") ) {
            setNickname(parameters[2], this);
          }


          //- show nicknames ------------------------------------------------------------------
          if ( parameters.length == 2 && parameters[0].equals("show") && parameters[1].equals("nicknames") ) {
            this.output( VoE_SessionComm.showNicknames() + "\r\n" );
          }


          //- set peer $nickname --------------------------------------------------------------
          if ( parameters.length == 3 && parameters[0].equals("set") && parameters[1].equals("peer") ) {
            this.setPeerName(parameters[2]);
          }


          //- start echoserver ----------------------------------------------------------------
          if ( parameters.length == 4 && parameters[0].equals("start") && parameters[1].equals("echoserver") ) {

            if (parameters[3].equals("tcp") || parameters[3].equals("udp")) {

              if (VoE_SessionComm.startEchoServer(parameters[2], parameters[3], nickname)) {
                this.output("echoServer started on "+parameters[2]+"/"+parameters[3]+"\r\n\r\n");
                this.echoServers.addElement(parameters[2]+"/"+parameters[3]);
              } else {
                this.output("error starting server\r\n\r\n");
              }

            } else {
              this.output("protocol is invalid\r\n\r\n");
            }

          }


          //- stop echoserver -----------------------------------------------------------------
          if ( parameters.length == 4 && parameters[0].equals("stop") && parameters[1].equals("echoserver") ) {

            if (parameters[3].equals("tcp") || parameters[3].equals("udp")) {

              if (VoE_SessionComm.stopEchoServer(parameters[2], parameters[3], nickname)) {
                this.output("echoServer stopped on "+parameters[2]+"/"+parameters[3]+"\r\n\r\n");
                this.echoServers.remove(parameters[2]+"/"+parameters[3]);
              } else {
                this.output("error stopping server\r\n\r\n");
              }

            } else {
              this.output("protocol is invalid\r\n\r\n");
            }

          }



// leave session


//          clOut.flush();

//          int n;
//          while ((n = srIn.read(srBuf)) > 0) {
//            clOut.write(srBuf,0,n);
//          }

        } while (clSocket.isConnected() && !request.equals("quit"));

        if (clSocket.isConnected()) {
          clOneOut.writeBytes("\r\nbye bye...\r\n\r\n");
        }

        System.out.println("VoE Thread("+threadCounter+")        \t--> closing connection to "+clSocket.getInetAddress().getHostAddress()+", port "+clSocket.getPort());
        cleanup();
        clSocket.close();

      } catch (IOException e) {
        System.out.println("VoE Thread("+threadCounter+")        \t--> closing connection to "+clSocket.getInetAddress().getHostAddress()+", port "+clSocket.getPort()+" because of an IOException");
        cleanup();
        clSocket.close();
      }


    }

}

