package VoE;

/* (k) 2005 by ahzf voe@ahzf.de */

import java.io.*;
import java.net.*;
import java.util.*;
import java.util.StringTokenizer;
import java.util.List;
import java.util.Iterator;
import java.util.Timer;
import java.util.HashMap;
import java.io.File;
//import VoE_UDPEchoServer;



class VoE_CommServer_SessionComm extends VoE_CommServer implements Runnable {


    final  static int    BUF_SIZE = 65000;
    static final  byte[] EOL      = {(byte)'\r', (byte)'\n' };

    /* buffer to use for requests */
    byte[]			clBuf;
    byte[]			srBuf;
    HashMap			nicknames, peerings, echoServers;
    VoE_CommServerThread	threadOne, threadTwo;
    int				echoServerThreadCounter;
    long			sleepTime;




    VoE_CommServer_SessionComm() {
      clBuf			= new byte[BUF_SIZE];
      srBuf			= new byte[BUF_SIZE];
      nicknames			= new HashMap();
      peerings 			= new HashMap();
      echoServerThreadCounter	= 0;
      echoServers		= new HashMap();
      sleepTime			= 20000;
    }



    public boolean registerNickname(String nickname, VoE_CommServerThread thread) {

      try {
        synchronized (nicknames) {

          if (nicknames.get(nickname) == null) {
            nicknames.put(nickname, thread);
            return(true);
          } else {
            return(false);
          }

        }
      } catch (Exception e) {
        return(false);
      }

    }



    public String[] getNicknames() {

      String[] nix = {""};

      try {
        String[] nicks = (String[]) nicknames.keySet().toArray(nix);
        return(nicks);
      } catch (Exception e) {
        nix[0] = e.toString();
        return(nix);
      }

    }



    public String showNicknames() {

      String   nicks		= "";
      String[] nickArray	= getNicknames();

      for (int i=0;i<nickArray.length;i++) {
        nicks = nicks + nickArray[i]+"\r\n";
      }

      return(nicks);

    }



    public String[] showPeerings() {

      String[] nix = {""};

      try {

        String[] peers = (String[]) peerings.keySet().toArray(nix);

        System.out.println("peerings:");
        for (int i=0;i<peers.length;i++) {
          System.out.println(peers[i]+" -> "+peerings.get(peers[i]));
        }

        return(nix);

      } catch (Exception e) {
        nix[0] = e.toString();
        return(nix);
      }

    }



    public boolean isValidPeer(String name) {
      synchronized (nicknames) {
        if (nicknames.get(name) == null) {
          return(false);
        } else {
          return(true);
        }
      }
    }



    public void requestPeering(String nickname, String peername) {

      try {

        if (peerings.get(nickname) == null) peerings.put(nickname, peername);

        String oneS = (String) peerings.get(nickname);
        String twoS = (String) peerings.get(peername);

        if (oneS != null && twoS != null && oneS.equals(peername) && twoS.equals(nickname) ) {
          VoE_CommServerThread oneT = (VoE_CommServerThread) nicknames.get(nickname);
          VoE_CommServerThread twoT = (VoE_CommServerThread) nicknames.get(peername);
          oneT.setPeer(twoT);
          twoT.setPeer(oneT);
        }

      } catch (Exception e) {
        System.out.println(e);
      }

    }



    public void removeNickname(String nickname) {


      synchronized (peerings) {

//        showPeerings();

        if (peerings.containsKey(nickname) && peerings.containsValue(nickname)) {
          String               peerName	= (String)               peerings.get(nickname);
          VoE_CommServerThread peer	= (VoE_CommServerThread) nicknames.get(peerName);
          peer.output("peer "+nickname+" closed connection to commServere\r\n");
          peer.disablePeering();
          peerings.remove(nickname);
        }

      }


      synchronized (echoServers) {

        Iterator it = echoServers.entrySet().iterator();

        while(it.hasNext()) {
          Map.Entry me = (Map.Entry)it.next();
          VoE_EchoServerThread echoServer = (VoE_EchoServerThread) me.getValue();
          echoServer.stopServer(nickname);
        }

      }


      synchronized (nicknames) {
        nicknames.remove(nickname);
      }


    }




    public boolean startEchoServer(int port, String protocol, String user) {
      return(startEchoServer(String.valueOf(port), protocol, user));
    }



    public boolean startEchoServer(String port, String protocol, String user) {

      String			serverId   = port.toString()+"/"+protocol;
      int			portInt    = 0;
      VoE_EchoServerThread	echoServer = null;
//      VoE_TCPEchoServerThread	TCPEchoServer = null;

      try {

        portInt = Integer.parseInt(port);

        if (echoServers.get(serverId) == null) {

          if (protocol.toLowerCase().equals("udp")) {
            echoServer = (VoE_EchoServerThread) new VoE_UDPEchoServerThread(echoServerThreadCounter, portInt, user);
          } else if (protocol.toLowerCase().equals("tcp")) {
            echoServer = (VoE_EchoServerThread) new VoE_TCPEchoServerThread(echoServerThreadCounter, portInt, user);
          }

          if (!echoServer.isValid()) echoServer = null;

          if (echoServer != null) {
            (new Thread(echoServer, "echoServer#"+echoServerThreadCounter++)).start();
            echoServers.put(serverId, echoServer);
            echoServer = null;
            return(true);
          } else {
            return(false);
          }

        } else {
          VoE_EchoServerThread runningServer = (VoE_EchoServerThread) echoServers.get(serverId);
          runningServer.addUser(user);
          runningServer = null;
          return(true);
        }

      } catch (NumberFormatException e) {
        return(false);
      }


    }




    public boolean stopEchoServer(String port, String protocol, String user) {

      String               serverId   = port.toString()+"/"+protocol;
      int                  portInt    = 0;
      VoE_EchoServerThread echoServer = null;

      try {

        portInt = Integer.parseInt(port);
        VoE_EchoServerThread runningServer = (VoE_EchoServerThread) echoServers.get(serverId);

        if (runningServer != null) {

          //- wenn der letzte user den echoserver beendet hat ----------
          if (runningServer.stopServer(user)) {
            System.out.println("VoE_CommServer_SessionComm() killing '"+serverId);
            runningServer.killServer();
            echoServers.remove(serverId);
            System.out.println("VoE_CommServer_SessionComm() removed '"+serverId+"' from echoServers hash");
          } else {
            System.out.println("VoE_CommServer_SessionComm() '"+serverId+"' still in use");
          }

          return(true);

        } else {
          return(false);
        }

      } catch (NumberFormatException e) {
        return(false);
      }


    }




//    public synchronized void run() {
    public void run() {

      System.out.println("VoE_CommServer_SessionComm() started");

      while(true) {
        try {
          Thread.sleep(sleepTime);
        } catch (Exception e) {
        }
      }

    }

}



