import java.net.*;
import java.math.*;
import java.io.*;
import java.util.Vector;
import java.util.Enumeration;


public class VoE_UDPEchoTest {

    
  public static void main(String args[]){ 
    
    // arguments to main coLgntain
    // Arg 1: the destination hostname or IP
    // Arg 2: the destination well-known port
    // Arg 3: how many repeats

    int			transmissionTimeout		= 1000;
    int			messagesize			= 1300;

    InetAddress		host;
    int			serverPort			= 2305;
    int			udpTestPort			= 2000;
    int			repeats				= 1000;

    DatagramSocket	udpTestSocket			= null;
    boolean		udpTestTimeout			= false;

    long		startDate			= System.currentTimeMillis();
    long		duration;
    long		sendingDate;

    Vector		rttVector			= new Vector();
    int			sendCounter			= 0;
    int			packetlostCounter		= 0;
    double		success				= 0;

    long		min				= 0;
    long		max				= 0;
    double		mean				= 0;
    double		variance			= 0;
    double		confidence			= 0;


    if (args.length != 4) {
      System.out.println("Usage: VoE_UDPEchoTest hostname serverPort udpTestPort repeats\n");
      System.exit(1);
    } else {

      try {

        host		= InetAddress.getByName(args[0]);
        serverPort	= Integer.parseInt(args[1]);
        udpTestPort	= Integer.parseInt(args[2]);
        repeats		= Integer.parseInt(args[3]);


        // create a datagram socket
        udpTestSocket = new DatagramSocket();
        udpTestSocket.setSoTimeout(transmissionTimeout);

        byte[] m			= new byte[65536];
        byte[] buffer			= new byte[65536];


        for (int i=0;i<repeats;i++) {

          udpTestTimeout		= false;

          // create a request and a reply datagram
          DatagramPacket request	= new DatagramPacket(m, messagesize, host, udpTestPort);
          DatagramPacket reply		= new DatagramPacket(buffer, buffer.length);


          // get the current time in milliseconds...	    
          sendCounter++;
          sendingDate = System.currentTimeMillis();
          udpTestSocket.send(request);


          try {
            udpTestSocket.receive(reply);
          } catch (IOException e) {
            udpTestTimeout = true;
            packetlostCounter++;
          }

          // calculate and store the round trip time
          if (!udpTestTimeout) {
            rttVector.addElement(System.currentTimeMillis()-sendingDate);
            System.out.print(".");
          }


        }


        //- calculate min, max and mean -------------------------
        min		= 2*transmissionTimeout;
        max		= 0;
        mean		= 0;


        Enumeration enumi = rttVector.elements();
        long	    value = 0;

        while (enumi.hasMoreElements()) {
          value = ( (Long) enumi.nextElement()).longValue();
          mean += value;
          if (value < min) { min = value; }
          if (value > max) { max = value; }
        }

        mean = mean/rttVector.size();



        //- calculate 95% standard derivation -------------------
        enumi = rttVector.elements();
        value = 0;

        while (enumi.hasMoreElements()) {
          value    = ( (Long) enumi.nextElement()).longValue();
          variance = (value - mean) * (value - mean);
        }

        variance	= Math.sqrt(variance/(rttVector.size()-1));
        confidence	= (1.96 * variance) / ( Math.sqrt( ((Integer) rttVector.size()).doubleValue()) );
        success		= ( ((Integer) rttVector.size()).doubleValue() ) / sendCounter*100;
        duration	= (System.currentTimeMillis() - startDate) / 1000;


        System.out.println("\n\nmeasurement took "+duration+"sec");
        System.out.println(sendCounter+" packets send; "+packetlostCounter+" packets lost ("+success+"% success)");
        System.out.println("min "+min+"ms, max "+max+"ms, mean "+mean+"ms +-"+confidence+"ms");


      } catch (SocketException e) {
        System.out.println("Socket: " + e.getMessage());

      } catch (IOException e) {
        System.out.println("IO: " + e.getMessage());

      } finally {
        if(udpTestSocket != null) udpTestSocket.close();
      }


    }

  }

}
